
#include "cover.h"


Cover::Cover ( const float scale, const float maxradius )
  : m_scale( scale ),
    m_gridorigin( m_scale * 0.25, m_scale * SIN60 * 0.5 ), // offset most properly for sections originating at 0,0
    m_sectionheight( ( m_scale * SIN60 ) * floor( maxradius / ( m_scale * SIN60 ) + 2 ) ), // + 2 for luck
    m_sectionwidth( m_sectionheight / SIN60 )
{}


Cover::~Cover ()
{
  sectmap_t::const_iterator sect;
  sect = m_sections.begin();
  while( sect != m_sections.end() )
  {
    delete sect->second;
    sect++;
  }
}


void Cover::cover ( const circle & area, const unsigned int id )
{
  std::set< CoverSection * > sections = getSections( area );
  std::set< CoverSection * >::const_iterator sect;
  sect = sections.begin();
  while( sect != sections.end() )
  {
    m_score -= (*sect)->getScore();
    (*sect)->cover( area, id );
    m_score += (*sect)->getScore();
    sect++;
  }
}


void Cover::uncover ( const circle & area, const unsigned int id )
{
  std::set< CoverSection * > sections = getSections( area );
  std::set< CoverSection * >::const_iterator sect;
  sect = sections.begin();
  while( sect != sections.end() )
  {
    m_score -= (*sect)->getScore();
    (*sect)->uncover( area, id );
    m_score += (*sect)->getScore();
    sect++;
  }
}


void Cover::uncoverAll ()
{
  sectmap_t::const_iterator sect;
  sect = m_sections.begin();
  while( sect != m_sections.end() )
  {
    sect->second->uncoverAll();
    sect++;
  }
  reinitScore();
}


void Cover::setTarget ( const circle & area, const bool target )
{
  std::set< CoverSection * > sections = getSections( area );
  std::set< CoverSection * >::const_iterator sect;
  sect = sections.begin();
  while( sect != sections.end() )
  {
    m_score -= (*sect)->getScore();
    (*sect)->setTarget( area, target );
    m_score += (*sect)->getScore();
    sect++;
  }
}


void Cover::unTargetAll ()
{
  sectmap_t::const_iterator sect;
  sect = m_sections.begin();
  while( sect != m_sections.end() )
  {
    sect->second->unTargetAll();
    sect++;
  }
  reinitScore();
}


float Cover::targetArea () const
{
  float area = 0;
  sectmap_t::const_iterator sect;
  sect = m_sections.begin();
  while( sect != m_sections.end() )
  {
    area += sect->second->targetArea();
    sect++;
  }
  return area;
}


const CoverScore & Cover::getScore () const
{
	return m_score;
}


void Cover::drawCells ( Brush & brush ) const
{
  sectmap_t::const_iterator sect;
  sect = m_sections.begin();
  while( sect != m_sections.end() )
  {
    sect->second->drawCells( brush );
    sect++;
  }
}


void Cover::drawPoints ( Brush & brush ) const
{
  sectmap_t::const_iterator sect;
  sect = m_sections.begin();
  while( sect != m_sections.end() )
  {
    sect->second->drawPoints( brush );
    sect++;
  }
}


void Cover::drawSections ( Brush & brush ) const
{
  sectmap_t::const_iterator sect;
  sect = m_sections.begin();
  while( sect != m_sections.end() )
  {
    const rect_f area = sect->second->getBound();
    brush.setRGBA( 0.1, 0.1, 0.8, 0.9 );
    brush.drawBox( area );
    sect++;
  }
}


void Cover::markCells ( Brush & brush, const circle & mark )
{
  std::set< CoverSection * > marked = getSections( mark );
  sectmap_t::const_iterator sect;
  sect = m_sections.begin();
  while( sect != m_sections.end() )
  {
    if( marked.find( sect->second ) != marked.end() )
      sect->second->markCells( brush, mark );
    sect++;
  }
}


CoverSection * Cover::getSection ( const vec_2i & coord )
{
  const int offset = (int)1 << 15;
  const unsigned int hashkey = ( ( ( coord.x + offset ) & 0xffff ) << 16 ) | ( ( coord.y + offset ) & 0xffff );

  /*unsigned int k = hashkey;
  for( int i = 0; i < 32; i++ )
  {
    if( k & 1 )
      cout << "1";
    else
      cout << "0";
    k = k >> 1;
  }
  cout << endl;
  cout.flush();*/

  sectmap_t::iterator sect = m_sections.find( hashkey );
  if( sect != m_sections.end() )
    return sect->second;
  else
  {
    const float left = coord.x * m_sectionwidth;
    const float right = ( coord.x + 1 ) * m_sectionwidth;
    const float bottom = coord.y * m_sectionheight;
    const float top = ( coord.y + 1 ) * m_sectionheight;
    const rect_f area( left, right, top, bottom );

    CoverSection * newsect = new CoverSection( area, m_scale, m_gridorigin );
    m_sections.insert( std::pair< unsigned int, CoverSection * >( hashkey, newsect ) );
    m_score += newsect->getScore();
    return newsect;
  }
}


CoverSection * Cover::getSection ( const vec_2f & point )
{
  const vec_2i coord( fastfloor( point.x / m_sectionwidth ),
                      fastfloor( point.y / m_sectionheight ) );
  return getSection( coord );
}


std::set< CoverSection * > Cover::getSections ( const circle & area )
{
  std::set< CoverSection * > sections;

  const vec_2i center = vec_2i( fastfloor( area.center().x / m_sectionwidth ),
                                fastfloor( area.center().y / m_sectionheight ) );
  const int wideradius = fastfloor( area.radius() / m_sectionwidth + 1 );
  const int highradius = fastfloor( area.radius() / m_sectionheight + 1 );
  for( int x = center.x - wideradius; x <= center.x + wideradius; x++ )
    for( int y = center.y - highradius; y <= center.y + highradius; y++ )
    {
      CoverSection * sect = getSection( vec_2i( x, y ) );
      if( area.intersects( sect->getBound() ) )
        sections.insert( sect );
    }

  return sections;
}


void Cover::reinitScore ()
{
	m_score.clear();
  sectmap_t::const_iterator sect;
  sect = m_sections.begin();
  while( sect != m_sections.end() )
  {
    m_score += sect->second->getScore();
    sect++;
  }
}
