#include "view.h"

SDLGLview::SDLGLview ( int w, int h )
{
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

  cur_focus = vec_2f( 0, 0 );
  cur_scale = 1.0;

  drag_base_world = target_focus = cur_focus;
  drag_base_view = vec_2f( 0, 0 );
  target_scale = cur_scale;
  pan_rate = 0.0;

  screen = NULL;
	resizeScreen( w, h );
}


SDLGLview::~SDLGLview ()
{
	SDL_FreeSurface( screen );
}


SDL_Surface * SDLGLview::getScreen ()
{
	return screen;
}


void SDLGLview::resizeScreen ( int w, int h )
{
	if( screen != NULL )
		SDL_FreeSurface( screen );
	viewport_w = w;
	viewport_h = h;
	screen = SDL_SetVideoMode( viewport_w, viewport_h, 0, SDL_OPENGL | SDL_RESIZABLE );
	if( screen == NULL )
		throw( "sdglview: failed to reallocate screen" );

  glEnable( GL_BLEND );
  glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
  glHint( GL_LINE_SMOOTH_HINT, GL_DONT_CARE );
  glHint( GL_POINT_SMOOTH_HINT, GL_DONT_CARE );
  glHint( GL_POLYGON_SMOOTH_HINT, GL_DONT_CARE );
  glEnable( GL_LINE_SMOOTH );
  glEnable( GL_POINT_SMOOTH) ;
  glEnable( GL_POLYGON_SMOOTH );

  glViewport( 0, 0, viewport_w, viewport_h );
  updateProjection();
  fill( 0.0, 0.0, 0.0, 1.0 );
}


void SDLGLview::resizeScreen ( const SDL_ResizeEvent & resizeevent )
{
  resizeScreen( resizeevent.w, resizeevent.h );
}


void SDLGLview::setCam ( const vec_2f & focus, float scale )
{
	cur_focus = target_focus = focus;
	cur_scale = target_scale = scale;
  updateProjection();
}


void SDLGLview::setCam ( const rect_f & area, float minscale )
{
	float scale = minscale;
	if( area.height() != 0 && area.width() != 0 ){
		scale = viewport_h / area.height() < scale ? viewport_h / area.height() : scale;
		scale = viewport_w / area.width() < scale ? viewport_w / area.width() : scale;
	}
	setCam( area.center(), scale );
}


void SDLGLview::setDragBase ( const vec_2f & viewbase )
{
	drag_base_view = viewbase;
	drag_base_world = cur_focus;
}


void SDLGLview::dragCam ( const vec_2f & topoint, float rate )
{
	panCam( drag_base_world - ( topoint - drag_base_view ) / cur_scale, rate );
}


void SDLGLview::panCam ( const vec_2f & target, float rate )
{
	target_focus = target;
	//target_scale = cur_scale;
	pan_rate = rate;
	pan_rate = pan_rate < 0.0 ? 0.0 : pan_rate;
	pan_rate = pan_rate > 1.0 ? 1.0 : pan_rate;
}


float SDLGLview::getTargetScale () const
  { return target_scale; }


void SDLGLview::panScale ( float scale, float rate )
{
	//target_focus = cur_focus;
	target_scale = scale;
	pan_rate = rate;
	pan_rate = pan_rate < 0.0 ? 0.0 : pan_rate;
	pan_rate = pan_rate > 1.0 ? 1.0 : pan_rate;
}


void SDLGLview::panTick ()
{
	if( pan_rate == 0.0 )
    return;
  // check if it's time to freeze it
  bool freeze = false;
  freeze &= ( cur_focus - target_focus ).magnitude() < 0.003;
  freeze &= fabsf( (cur_scale - target_scale) / cur_scale ) < 0.003;
  if( freeze ){
    cur_focus = target_focus;
    cur_scale = target_scale;
    pan_rate = 0.0;
  }
  else
  {
    cur_focus = target_focus * pan_rate + cur_focus * ( 1.0 - pan_rate );
    cur_scale = target_scale * pan_rate + cur_scale * ( 1.0 - pan_rate );
  }
  updateProjection();
}


float SDLGLview::getScale () const
  { return cur_scale; }


// TODO move this into a generic drawing gizmo class
void SDLGLview::fill ( float r, float g, float b, float a )
{
	if( a == 1.0 )
	{
		glClearColor( r, g, b, a );
		glClear( GL_COLOR_BUFFER_BIT );
	}
	else
	{
		glMatrixMode( GL_MODELVIEW );
    glPushMatrix();
    glLoadIdentity();
		glColor4f( r, g, b, a );
		glRectf(
			cur_focus.x - 0.5 * viewport_w / cur_scale,
			cur_focus.y - 0.5 * viewport_h / cur_scale,
			cur_focus.x + 0.5 * viewport_w / cur_scale,
			cur_focus.y + 0.5 * viewport_h / cur_scale );
    glPopMatrix();
	}
}


vec_2f SDLGLview::ViewToWorld ( const vec_2f view_point ) const
{
	vec_2f world_point( view_point );
	world_point -= vec_2f( viewport_w / 2, viewport_h / 2 );
	world_point /= cur_scale;
	world_point += cur_focus;
	return world_point;
}


void SDLGLview::updateProjection ()
{
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glOrtho( 0.0, (float)viewport_w, (float)viewport_h, 0.0, 0.0, 10.0 );
  glTranslatef( (cur_scale * -cur_focus.x + viewport_w / 2),
  							(cur_scale * -cur_focus.y + viewport_h / 2), 0.0 );
  glScalef( cur_scale, cur_scale, 1.0 );
}
