
#include "sensor.h"


Sensor::Sensor ( const vec_2f & startpos, const float mobility, const float radius, const unsigned int id )
	: m_startpos( startpos ), m_mobility( mobility ), m_radius( radius ), m_id( id )
{
	scramble();
}


void Sensor::draw ( Brush & brush, const bool paths, float progress ) const
{
	MWCRand rnd;
	rnd.seed( (unsigned int)this );

	if( paths )
	{
	  brush.setRGBA( 0.0, 0.0, 0.0, 0.5 );
	  brush.drawLineStrip( getPath() );
	}

  vec_2f pos = m_startpos;
  if( progress > 0.0 && progress <= 1.0 )
  {
    float dist = m_mobility * progress;
    unsigned int segi = 0;
    while( segi < m_path.size() )
    {
      const vec_2f & segment = m_path[ segi ];
      const float seglen = segment.magnitude();
      if( dist < seglen )
      {
        pos += segment * ( dist / seglen );
        break;
      }
      else
      {
        pos += segment;
        dist -= seglen;
        segi++;
      }
    }
  }

  brush.setRGBA( 0.1, 0.2, 0.5 + 0.2 * rnd.rand(), 1.0 );
  brush.drawSpot( pos, 0.4 );
  brush.setRGBA( 0.0, 0.0, 0.0, 0.2 );
  brush.drawCircle( circle( pos, m_radius ) );
}


void Sensor::scramble ()
{
  float mobility = m_mobility;
  m_path.clear();
  while( mobility > 0 )
  {
    vec_2f v;
    v.setnormal( mt.rand() * TWOPI );
    if( mobility > m_radius )
    {
      v *= m_radius;
      mobility -= m_radius;
    }
    else
    {
      v *= mobility;
      mobility = 0;
    }
    m_path.push_back( v );
  }
}


void Sensor::scrambleStraight ()
{
  float mobility = m_mobility;
  m_path.clear();
  vec_2f v;
  v.setnormal( mt.rand() * TWOPI );
  while( mobility > 0 )
  {
    if( mobility > m_radius )
    {
      m_path.push_back( v * m_radius );
      mobility -= m_radius;
    }
    else
    {
      m_path.push_back( v * mobility );
      mobility = 0;
    }
  }
}


void Sensor::mutate ( const float anglevar )
{
  const float turnchance = 0.2;
  const float swapchance = 0.2;

  if( m_path.size() < 1 )
    return;

  if( m_path.size() > 1 && mt.rand() < swapchance )
  {
    unsigned int i1 = mt.randInt( m_path.size() - 2 );
    unsigned int i2 = mt.randInt( m_path.size() - 2 );
    vec_2f t = m_path[ i1 ];
    m_path[ i1 ] = m_path[ i2 ];
    m_path[ i2 ] = t;
  }
  else
  {
    float angle = mt.randNormf( 0.0, anglevar );
    unsigned int secti = mt.randInt( m_path.size() - 1 );
    m_path[ secti ] = m_path[ secti ].rotate( angle );
    if( mt.rand() < turnchance )
    {
      angle *= 2.0;
      while( secti < m_path.size() )
      {
        m_path[ secti ] = m_path[ secti ].rotate( angle );
        secti++;
      }
    }
  }
}


void Sensor::setRawPath ( const std::vector< vec_2f > & newpath )
{
  m_path = newpath;
  // TODO error checking here
}


std::vector< vec_2f > Sensor::getPath () const
{
  vec_2f pos = m_startpos;
  std::vector< vec_2f > path;
  path.push_back( pos );
  for( unsigned int i = 0; i < m_path.size(); i++ )
  {
    pos += m_path[ i ];
    path.push_back( pos );
  }
  return path;
}


float Sensor::maxArea () const
{
  return m_radius * m_radius * TWOPI * 0.5 +
         m_radius * 2.0 * m_mobility;
}
