
#include "simulation.h"

Simulation::Simulation ( const float senseradius )
	: m_swarm( senseradius )
{
  m_time = 0;
	m_covern = 0;
}

Simulation::~Simulation () {}


void Simulation::draw ( Brush & brush, const drawopts_t & opts ) const
{
	if( opts.cover )
		m_swarm.drawCover( brush, m_covern );
	if( opts.sensors )
	{
	  float t = m_time;
	  if( t > 1.0 )
      if( t < 1.5 )
        t = 1.0;
      else
        if( t < 1.7 )
          t = 1.0 - ( t - 1.5 ) / 0.2;
        else
          t = 0.0;
    m_swarm.drawSensors( brush, opts.paths, t );
	}
}


void Simulation::resetTime ()
  { m_time = 0.0; }


void Simulation::step ( const float step )
{
	m_time += step;
	if( m_time > 2.0 )
		m_time = 0.0;
}


void Simulation::addCluster ( const Sensor & base, const vec_2f & center, const float radius, const unsigned int count )
{
	for( unsigned int n = 0; n < count; n++ )
	{
		float r = mt.rand();
		r = r * radius;
		vec_2f pos;
		pos.setnormal( mt.rand() * TWOPI );
		pos *= r;
    pos += center;
    Sensor s( pos, base.getMobility(), base.getRadius(), mt.randInt() );
		m_swarm.addSensor( s );
	}
}


void Simulation::addUniform ( const Sensor & base, const rect_f & area, const unsigned int count )
{
	for( unsigned int i = 0; i < count; i++ )
	{
		vec_2f pos( area.left() + mt.rand() * area.width(),
                area.bottom() + mt.rand() * area.height() );
    Sensor s( pos, base.getMobility(), base.getRadius(), mt.randInt() );
		m_swarm.addSensor( s );
	}
}


void Simulation::setTarget ( const circle & area, const bool target )
{
  m_swarm.setTarget( area, target );
}


std::vector< float > Simulation::getTargetNCover () const
{
  const CoverScore & score = m_swarm.getCover().getScore();
  const std::vector< unsigned int > & tcover = score.targetCover();
  std::vector< float > data;
  data.resize( tcover.size(), 0 );
  for( unsigned int i = 0; i < tcover.size(); i++ )
  {
    data[ 0 ] += tcover[ i ];
    if( i > 0 )
      data[ i ] += tcover[ i ];
  }
  if( data.size() < 1 )
    return data;
  for( unsigned int i = tcover.size() - 1; i > 1; i-- )
    data[ i - 1 ] += data[ i ];
	return data;
}


std::vector< float > Simulation::getMaxNCover () const
{
  float coverarea = m_swarm.maxArea();
  float targetarea = m_swarm.getCover().targetArea();
  std::vector< float > data;

  data.push_back( targetarea );
  if( coverarea == 0 || targetarea == 0 )
  {
    data.push_back( 0 );
    return data;
  }
  while( coverarea > targetarea )
  {
    data.push_back( targetarea );
    coverarea -= targetarea;
  }
  data.push_back( coverarea );
  while( data.size() > m_swarm.sensorCount() + 1 )
    data.pop_back();
  return data;
}


unsigned int Simulation::N () const
  { return m_covern; }


void Simulation::setN ( const unsigned int n )
  {	m_covern = n; }


void Simulation::scramble ()
{
	m_swarm.scramble();
	m_its = 0;
}


void Simulation::optimizeN ()
{
	m_its += m_swarm.sensorCount();
	m_swarm.setTargetNCover( m_covern );
	m_swarm.mutateAllOnce();
}


void Simulation::optimize ( unsigned int iterations )
{
	m_its += iterations;
	m_swarm.setTargetNCover( m_covern );
	while( iterations-- > 0 )
		m_swarm.mutateOneRand();
}


unsigned int Simulation::totalIts () const
  {	return m_its; }


void Simulation::clearSensors ()
{
	m_its = 0;
	m_swarm.clear();
}


void Simulation::clearTarget ()
{
	m_swarm.unTargetAll();
}
